from analytics_ingest.internal.schemas.inputs.message_input import (
    make_create_message_input,
)
from analytics_ingest.internal.schemas.message_schema import MessageSchema
from analytics_ingest.internal.utils.graphql_executor import GraphQLExecutor
from analytics_ingest.internal.utils.mutations import GraphQLMutations

_message_cache = {}


def get_cached_message_id(key: str) -> str | None:
    return _message_cache.get(key)


def create_message(executor: GraphQLExecutor, variables: list[dict]) -> list[str]:
    message_ids = []
    uncached_messages = []
    comparison_keys = []

    for var in variables:
        message = MessageSchema.from_variables(var)
        key = message.cache_key()

        if key in _message_cache:
            message_ids.append(_message_cache[key])
        else:
            input_dict = {
                "arbId": var.get("arbId"),
                "name": var.get("name"),
                "networkName": var.get("networkName"),
                "ecuName": var.get("ecuName"),
                "ecuId": var.get("ecuId"),
                "requestCode": None,
                "fileId": var.get("fileId"),
                "messageDate": var.get("timestamp"),
            }

            comparison_keys.append(key)
            uncached_messages.append(input_dict)

    if uncached_messages:
        response = executor.execute(
            GraphQLMutations.create_message(),
            make_create_message_input(uncached_messages),
        )

        messages = response["data"].get("createMessage", [])
        if not messages:
            raise RuntimeError("No messages created")

        response_map = {
            (msg["arbId"], msg["name"], msg["networkName"], msg["ecuName"]): msg["id"]
            for msg in messages
        }

        for idx, input_dict in enumerate(uncached_messages):
            key = comparison_keys[idx]
            compare_tuple = (
                input_dict["arbId"],
                input_dict["name"],
                input_dict["networkName"],
                input_dict["ecuName"],
            )
            msg_id = response_map.get(compare_tuple)
            if msg_id:
                _message_cache[key] = msg_id
                message_ids.append(msg_id)

    return message_ids
